#include "StdAfx.h"
#include "RecentChatListManager.h"
#include "Resource.h"
#include "ChatDialogManager.h"
#include "main_frame.hpp"
#include "ChatDlg.h"
#include "ChatModel.h"
#include "SendNoteMananger.h"
#include "NeMessengerDlg.h"
#include "UnReadChatMessage.h"

////////////////Begin of CDuiOrgList implement////////////////////
#define TIME_REMOVE_CHAT	(7*24*60*60)

std::vector<Node*> CDuiOrgList::GetSelectedItems()
{
	COrganizationTreeUI* pOrgList = static_cast<COrganizationTreeUI*>(m_pFrame->GetPaintManager()->FindControl(kOrganization));
	return pOrgList->GetSelectedItems();
}

CDuiOrgList::~CDuiOrgList()
{
	
}

////////////////Begin of CRecentChatListManager class implement///////////////
CRecentChatListManager::CRecentChatListManager(MainFrame* pFrame)
{
	m_pFrame = pFrame;
	m_pChattingRoomList = NULL;
}

CRecentChatListManager::~CRecentChatListManager()
{
	list<ChatRoomItem*>::iterator it = m_listChatItem.begin();
	for (; it!=m_listChatItem.end(); ++it)
	{
		delete (*it);
	}
	m_listChatItem.clear();
}

void CRecentChatListManager::InitChatRoomList(CChatRoomList* pChattingRoomList)
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	m_pChattingRoomList = pChattingRoomList;
	//ߺm_listChatItemᱻգҪChatDialogManagerݳʼm_listChatItem
	GET_APP()->m_pChatDialogManager->InitChatRoomList();
}

void CRecentChatListManager::OnRButtonEvent( bool bDown, int nIndex )
{
	const list<Node*>& SelectRooms = GetSelectedItems();

	if (SelectRooms.size() < 1) //select none
		return;

	CPoint pt(0, 0);
	GetCursorPos(&pt);
	CMenu menuList;
	menuList.LoadMenu(IDR_MEMU_COMMON_MENU);

	int nSubMenu = 6;
	CMenu *subMenu = menuList.GetSubMenu(nSubMenu);

	if (SelectRooms.size() > 1)
	{
		//can not send to multi chating user/group
		subMenu->EnableMenuItem(ID_CHAT_ROOM_SEND_MINI_MAIL,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
		subMenu->EnableMenuItem(ID_CHAT_ROOM_SEND_FILE,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
		subMenu->EnableMenuItem(ID_CHAT_ROOM_SEND_MAIL,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
		subMenu->EnableMenuItem(ID_CHAT_ROOM_SMS,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
	}

	int nOnlineUserCount = 0; 
	int nTotalUserCount = 0;     

	for (list<Node*>::const_iterator it = SelectRooms.begin(); it != SelectRooms.end(); it++)
	{
		int nRoomID = (*it)->GetCompareData();
		ChatRoomItem* pChatRoomItem = GetChatRoomItem(nRoomID);
		if (pChatRoomItem != NULL)
		{
			if (pChatRoomItem->pChatDlg == NULL)
			{
				nOnlineUserCount += pChatRoomItem->GetOnlineUserCount();
				nTotalUserCount += pChatRoomItem->GetTotalUserCount();
			}
			else
			{
				nOnlineUserCount += pChatRoomItem->pChatDlg->GetChatModel()->GetOnlineUserCount();
				nTotalUserCount += pChatRoomItem->pChatDlg->GetChatContactCntExceptMe();

				if (pChatRoomItem->pChatDlg->GetForceLeave())
				{
					subMenu->EnableMenuItem(ID_CHAT_ROOM_SEND_FILE,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED); 
				}
			}
		}
	}

	if (0 == nOnlineUserCount)
	{
		subMenu->EnableMenuItem(ID_CHAT_ROOM_SEND_FILE,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED); 
	}

	if (0 == nTotalUserCount)
	{
		subMenu->EnableMenuItem(ID_CHAT_ROOM_SEND_MINI_MAIL,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
		subMenu->EnableMenuItem(ID_CHAT_ROOM_SEND_MAIL,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
		subMenu->EnableMenuItem(ID_CHAT_ROOM_SMS,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
	}

#ifdef _DEBUG
#else
	if(!GET_CONFIG()->EnableFunction(_T("ME_01")))
	{
		subMenu->EnableMenuItem(ID_CHAT_ROOM_SMS, MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
	}
#endif
	int nCmdID = subMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_RETURNCMD | TPM_RIGHTBUTTON, pt.x, pt.y, CWnd::FromHandle(m_pFrame->GetSafeHwnd()));

	MenuCmd(nCmdID);
}

void CRecentChatListManager::SetItemChatDlg(int nRoomID, CChatDlg *pChatDlg)
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	list<ChatRoomItem*>::iterator it = m_listChatItem.begin();
	for (; it!=m_listChatItem.end(); ++it)
	{
        ChatRoomItem* pItem = *it;
		if (pItem->nRoomID == nRoomID)
		{
            ASSERT((NULL == pChatDlg) || (NULL == pItem->pChatDlg) || (pChatDlg == pItem->pChatDlg)); // avoid memory leak
			(*it)->pChatDlg = pChatDlg;
			break;
		}
	}
}

ChatRoomItem *CRecentChatListManager::GetChatRoomItem(int nRoomID)
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	ChatRoomItem *pChatRoomItem = NULL;
	list<ChatRoomItem*>::iterator it = m_listChatItem.begin();
	for (; it!=m_listChatItem.end(); ++it)
	{
		if ((*it)->nRoomID == nRoomID)
		{
			pChatRoomItem = (*it);
			break;
		}
	}
	return pChatRoomItem;
}

CChatDlg* CRecentChatListManager::RevertChatDialog(int nRoomID)
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	CChatDlg* pChatDlg = NULL;
	ChatRoomItem *pChatRoomItem = GetChatRoomItem(nRoomID);
	if (pChatRoomItem != NULL)
	{
		if (pChatRoomItem->pChatDlg == NULL)
		{
			NE_MYROOM_DATA* pRoomData = GET_APP()->m_pChatDialogManager->GetRoomData(nRoomID);
			if (pRoomData != NULL)
			{
				pChatDlg = GET_APP()->m_pChatDialogManager->CreateJoinedChat(*pRoomData);
				SetItemChatDlg(nRoomID, pChatDlg);
			}
		}
		else
		{
			pChatDlg = pChatRoomItem->pChatDlg;
		}
	}
	ASSERT(pChatDlg != NULL);
	return pChatDlg;
}

void CRecentChatListManager::OpenChatDialog(const list<Node*> &SelectRooms)
{
	list<Node*>::const_iterator it = SelectRooms.begin();
	for (; it != SelectRooms.end(); ++it)
	{
		int nRoomID = (*it)->GetCompareData();
		RevertChatDialog(nRoomID);
		GET_APP()->m_pChatDialogManager->SetForegroundChatDialog(nRoomID);
	}
}

void CRecentChatListManager::LeaveChatDialog(const list<Node*> &SelectRooms)
{
	CUIntArray nRoomIDArray;
	list<Node*>::const_iterator it = SelectRooms.begin();
	for (; it != SelectRooms.end(); ++it)
	{
		int nRoomID = (*it)->GetCompareData();
		nRoomIDArray.Add(nRoomID);
	}
	if (NMessageBox(CWnd::FromHandle(m_pFrame->GetSafeHwnd()), LoadStringFromResource(IDS_STRING348), _T("Ntalk biz"), MB_YESNO) == IDYES)
	{
		//GET_APP()->m_pChatDialogManager->SetNeedRestoreChatRoomID(false);
		CloseChatDialogs(nRoomIDArray);
	}
}

void CRecentChatListManager::CloseChatDialogs(const CUIntArray& nRoomIDArray)
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	for (int i = 0;i< nRoomIDArray.GetCount();i++)
	{
		int nRoomID = nRoomIDArray.GetAt(i);
		ChatRoomItem *pChatRoomItem = GetChatRoomItem(nRoomID);
		if (pChatRoomItem != NULL)
		{
			if (pChatRoomItem->pChatDlg == NULL)
			{
				NE_MYROOM_DATA* pRoomData = GET_APP()->m_pChatDialogManager->GetRoomData(nRoomID);
				if (pRoomData != NULL)
				{
					GT_MY_USER_DATA myUserData;
					g_pMessengerActor->GetMyUserInfo(myUserData);
					pChatRoomItem->RequestChatLeaveInRecentChatList(pRoomData->strServerIP.c_str(), pRoomData->nServerPort, pRoomData->nRoomID, myUserData.strUserID,
						myUserData.sMyPage.strUserName, myUserData.sMyPage.strTalkName);
				}
			}
			else
			{
				pChatRoomItem->pChatDlg->SetNeedRestoreChatRoom(false);
			}
		}
	}
	GET_APP()->m_pChatDialogManager->CloseChatDialogs(nRoomIDArray);
}

void CRecentChatListManager::RemoveChatRoomForTimeout()
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	CUIntArray nRoomIDArray;
	list<ChatRoomItem*>::iterator it = m_listChatItem.begin();
	for (; it!=m_listChatItem.end(); ++it)
	{
		CTime tmMsg = (*it)->tmLastTime;
		CTime tmNow = CTime::GetCurrentTime();
		GetLocaleTime(&tmNow);
		CTimeSpan tmSpan= tmNow - tmMsg;
		if (tmSpan.GetTotalSeconds() > TIME_REMOVE_CHAT)
		{
			nRoomIDArray.Add((*it)->nRoomID);
		}
	}
	if (nRoomIDArray.GetSize() > 0)
	{
		CloseChatDialogs(nRoomIDArray);
	}
}

void CRecentChatListManager::SendFileInChatDialog(const list<Node*> &SelectRooms)
{
	list<Node*>::const_iterator it = SelectRooms.begin();
	for (; it != SelectRooms.end(); ++it)
	{
		int nRoomID = (*it)->GetCompareData();
		CChatDlg *pChatDlg = RevertChatDialog(nRoomID);
		if (pChatDlg != NULL)
		{
			if (SENDACTION_FILE != pChatDlg->GetOngoingAction())					
			{
				::ShowWindow(pChatDlg->GetHWND(), SW_SHOW);
				pChatDlg->SetOngoingAction(SENDACTION_FILE);
				::PostMessage(pChatDlg->GetSafeHwnd(), WM_CHATDLGSENDFILE, 0, 0);
			}
		}
	}
}

void CRecentChatListManager::SendSMS(const list<Node*> &SelectRooms)
{
	vector<CSMSModel::SMSReceiverInfo> vecReceiveUsers;
	list<Node*>::const_iterator it = SelectRooms.begin();
	for (; it != SelectRooms.end(); ++it)
	{
		CString strUserID = (*it)->data().value;
		CString strName = (*it)->data().text_;
		if (strUserID.Find(SPLIT_STRING) != -1)
		{
			std::vector<CString> vecUserID;
			size_t nUserIDCount = Split(strUserID, SPLIT_STRING, false, vecUserID);
			std::vector<CString> vecUserName;
			size_t nUserNameCount = Split(strName, SPLIT_STRING, false, vecUserName);
			if (nUserIDCount == nUserNameCount)
			{
				for (size_t i=0; i<vecUserID.size(); ++i)
				{
					CSMSModel::SMSReceiverInfo receiveUser;
					receiveUser.strID = vecUserID[i];
					receiveUser.strName = vecUserName[i];
					const NE_MEMBER_DATA *data = g_pMessengerActor->GetEmployeeInfo(receiveUser.strID);
					if (data)
					{
						receiveUser.strCellPhone =  data->strCellPhone.c_str();
						vecReceiveUsers.push_back(receiveUser);
					}
				}
			}
		}
		else
		{
			CSMSModel::SMSReceiverInfo receiveUser;
			receiveUser.strID = (*it)->data().value;
			receiveUser.strName = (*it)->data().text_.GetData();
			const NE_MEMBER_DATA *data = g_pMessengerActor->GetEmployeeInfo(receiveUser.strID);
			if (data)
			{
				receiveUser.strCellPhone =  data->strCellPhone.c_str();
				vecReceiveUsers.push_back(receiveUser);
			}
		}
	}	
	GET_APP()->m_pSMSDialogManager->CreateListSMS(vecReceiveUsers);
}

void CRecentChatListManager::SendMiniMail(const list<Node*> &SelectRooms)
{
	vector<ReceiverInfo> vecReceiveUsers;
	list<Node*>::const_iterator it = SelectRooms.begin();
	for (; it != SelectRooms.end(); ++it)
	{
		CString strUserID = (*it)->data().value;
		CString strName = (*it)->data().text_;
		if (strUserID.Find(SPLIT_STRING) != -1)
		{
			std::vector<CString> vecUserID;
			size_t nUserIDCount = Split(strUserID, SPLIT_STRING, false, vecUserID);
			std::vector<CString> vecUserName;
			size_t nUserNameCount = Split(strName, SPLIT_STRING, false, vecUserName);
			if (nUserIDCount == nUserNameCount)
			{
				for (size_t i=0; i<vecUserID.size(); ++i)
				{
					ReceiverInfo receiveUser;
					receiveUser.strID = vecUserID[i];
					receiveUser.strName = vecUserName[i];
					vecReceiveUsers.push_back(receiveUser);
				}
			}
		}
		else
		{
			if (strUserID.GetLength() > 1)
			{
				ReceiverInfo receiveUser;
				receiveUser.strID = strUserID;
				receiveUser.strName = strName;
				vecReceiveUsers.push_back(receiveUser);
			}
		}
	}			
	GET_APP()->m_pSendNoteManager->CreateListNote(vecReceiveUsers);
}

void CRecentChatListManager::MenuCmd(int nCmdID)
{
	list<Node*> SelectRooms = GetSelectedItems();
	int nSelectedCount = SelectRooms.size();
	if (nSelectedCount <= 0)
		return;

	switch (nCmdID)
	{
	case ID_CHAT_ROOM_OPEN:
		{
			OpenChatDialog(SelectRooms);
		}
		break;

	case ID_CHAT_ROOM_CLOSE:
		{	
			LeaveChatDialog(SelectRooms);
		}
		break;
	case ID_CHAT_ROOM_SEND_FILE:
		{
			SendFileInChatDialog(SelectRooms);
		}
		break;
	case ID_CHAT_ROOM_SMS:
		{
			SendSMS(SelectRooms);
		}
		break;
	case ID_CHAT_ROOM_SEND_MINI_MAIL:
		{
			SendMiniMail(SelectRooms);		
		}
		break;
	default:
		break;
	}
}


list<Node*> CRecentChatListManager::GetSelectedItems()
{
	return m_pChattingRoomList->GetSelectedItems();
}

int CRecentChatListManager::GetCount()
{
	return m_pChattingRoomList->GetCount(); 
}

void CRecentChatListManager::AddAllChatRoomList()
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	if (m_pChattingRoomList != NULL)
	{
		vector<ChatRoomItem*> vecChatRoomItem;
		list<ChatRoomItem*>::iterator it = m_listChatItem.begin();
		for (; it!=m_listChatItem.end(); ++it)
		{
			if ((*it)->pChatDlg != NULL)
			{
				(*it)->CopyChatItem((*it)->pChatDlg);
			}
			vecChatRoomItem.push_back(*it);
		}
		if (!vecChatRoomItem.empty())
		{
			qsort_s(&(vecChatRoomItem.at(0)), vecChatRoomItem.size(), sizeof(ChatRoomItem*), ItemComareFunc, this);
		}
		for (size_t i=0; i<vecChatRoomItem.size(); ++i)
		{
			vecChatRoomItem[i]->UpdataChatItem();
			m_pChattingRoomList->AddChattingRoomNode(*(vecChatRoomItem[i]), NULL);
			if (vecChatRoomItem[i]->pChatDlg != NULL)
			{
				UpdateChatRoomList(vecChatRoomItem[i]->pChatDlg);
			}
		}

		ResortChatRoomList(vecChatRoomItem);
		UpdateAllChatRoomTime();
	}
}

int __cdecl CRecentChatListManager::ItemComareFunc(void *pvlocale,const void *elem1, const void *elem2)
{
	if (!pvlocale || !elem1 || !elem2)
		return 0;

	ChatRoomItem **pChatItem1 = (ChatRoomItem**)elem1;
	ChatRoomItem **pChatItem2 = (ChatRoomItem**)elem2;

	CTime time1 = (*pChatItem1)->tmLastTime;
	CTime time2 = (*pChatItem2)->tmLastTime;
	int nRet = 0;
	if(time1 > time2)
	{
		nRet = -1;
	}
	else
	{
		nRet = 1;
	}

	return nRet;
}

void CRecentChatListManager::ClearChatRoomList()
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	m_pChattingRoomList->RemoveAll();
}

BOOL CRecentChatListManager::DisplaySearchChatRoomList(CString strSearchText)
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	vector<CString> vecQueries;
	SplitString_NotEmpty(strSearchText.Trim(), _T(' '), vecQueries);
	int queryNo = vecQueries.size();	
	list<ChatRoomItem*>::iterator it = m_listChatItem.begin();
	for (; it!=m_listChatItem.end(); ++it)
	{
		CString strAllUserName = (*it)->strName;
		CString strAllUserNameLow = strAllUserName.MakeLower();

		//ȭ  ˻
		BOOL bSearchInUsers = FALSE;
		int nFind = 0;
		CString strQuery;
		for (int queryIndex = 0 ; queryIndex < queryNo ; ++queryIndex)
		{
			strQuery = vecQueries[queryIndex];
			CString strQueryLow = strQuery.MakeLower();
			if (FALSE == strQuery.IsEmpty() && -1 != strAllUserNameLow.Find(strQueryLow))
			{
				++nFind;
			}
			else
			{
				break;
			}
		}

		if (nFind == queryNo)
		{
			bSearchInUsers = TRUE;
		}
		if ((*it)->pChatDlg == NULL)
		{
			if ((*it)->IsInChatItem(strSearchText) == FALSE && !bSearchInUsers)
			{
				GET_MESSENGER_DLG()->GetMainFrame()->RemoveChatRoom((*it)->nRoomID, FALSE);
			}
			else
			{
				UpdateChatRoomList(*it);
			}
		}
		else
		{
			if ((*it)->pChatDlg->IsInChat(strSearchText) == FALSE && !bSearchInUsers)
			{
				GET_MESSENGER_DLG()->GetMainFrame()->RemoveChatRoom((*it)->nRoomID, FALSE);
			}
			else
			{
				(*it)->pChatDlg->GetCWnd()->RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_ALLCHILDREN);
				UpdateChatRoomList((*it)->pChatDlg);
			}
		}
	}

	int nItemCount = m_pChattingRoomList->GetCount();
	if (nItemCount == 0)
	{
		return FALSE;
	}

	return TRUE;
}

void CRecentChatListManager::RemoveChatRoom(int nRoomID, BOOL bDeleteData)
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	for(int i = 0; i < m_pChattingRoomList->GetRoot()->num_children(); i++)
	{
		Node* pNode = m_pChattingRoomList->GetRoot()->child(i);
		if(pNode->GetCompareData() == nRoomID)
		{
			m_pChattingRoomList->RemoveNode(pNode);
			break;
		}
	}
	if (bDeleteData)
	{
		list<ChatRoomItem*>::iterator it = m_listChatItem.begin();
		for (; it!=m_listChatItem.end(); ++it)
		{
			if ((*it)->nRoomID == nRoomID)
			{
				if ((*it)->pChatDlg != NULL)
				{
					delete (*it);
				}
				m_listChatItem.erase(it);
				break;
			}
		}
	}
}

int CRecentChatListManager::FindChatRoomIndex(int nRoomNo)
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);

    BOOL bFound = FALSE;
    int nIndex = -1;
	list<ChatRoomItem*>::iterator it = m_listChatItem.begin();
	for (; it!=m_listChatItem.end(); ++it)
	{
        nIndex++;
		if ((*it)->nRoomID == nRoomNo)
		{
            bFound = TRUE;
			break;
		}
	}

    if (bFound)
    {
        return nIndex;
    }
	return -1;
}

BOOL CRecentChatListManager::FindSingleChatDlg(LPCTSTR strUserID, int nRoomID)
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	BOOL bFind = FALSE;
	list<ChatRoomItem*>::iterator it = m_listChatItem.begin();
	for (; it!=m_listChatItem.end(); ++it)
	{
		const ChatRoomItem* pItem = *it;
		if (!pItem->bGroupChat && (pItem->strUserID == strUserID))
		{
			bFind = TRUE; 
			break;
		}
	}

	return bFind;
}

void CRecentChatListManager::UpdateAllChatRoomTime() // chatroomʱʾ
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	Node* pNode = NULL;
	int ChildIndx = 0;
	list<ChatRoomItem*>::iterator it;
	ChatRoomItem* pChatRoomItem = NULL;

	for (ChildIndx =0; ChildIndx < m_pChattingRoomList->GetRoot()->num_children(); ChildIndx ++)
	{
		pNode = m_pChattingRoomList->GetNodeByIndex(ChildIndx);
		int RoomID = pNode->GetCompareData();
		for (it=m_listChatItem.begin(); it!=m_listChatItem.end(); ++it)
		{
			if ((*it)->nRoomID == RoomID)
			{
				pChatRoomItem = *it;			
			}
		} 
		CLabelUI* date = static_cast<CLabelUI*>(m_pChattingRoomList->paint_manager_.FindSubControlByName(pNode->data().list_elment_, _T("date")));
		if((date != NULL) && (pChatRoomItem != NULL))
		{
			CString strTime = ParseChatTime(pChatRoomItem->tmLastTime);
			date->SetText(strTime);
		}
	}
}

void CRecentChatListManager::UpdateRecentUserStatus(IGTMessengerActor *pActor, LPCWSTR strUserID)
{
	list<ChatRoomItem*>::iterator it;
	for (it=m_listChatItem.begin(); it!=m_listChatItem.end(); ++it)
	{
		ChatRoomItem* pChatRoomItem = (*it);
		if (pChatRoomItem != NULL)
		{
			if ((!pChatRoomItem->bGroupChat) && (pChatRoomItem->strUserID == strUserID))
			{
				pChatRoomItem->UpdataChatItem();
				UpdateChatRoomList(pChatRoomItem);
			}
		}
	}
}

void CRecentChatListManager::UpdateChatRoomList(ChatRoomItem *pChatRoomItem)
{
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);

	vector<ChatRoomItem*> vecSortedChatItem;
	list<ChatRoomItem*>::iterator it;
	for (it=m_listChatItem.begin(); it!=m_listChatItem.end(); ++it)
	{
		vecSortedChatItem.push_back(*it);
	}
	if (vecSortedChatItem.size() > 1)
	{
		qsort_s(&(vecSortedChatItem.at(0)), vecSortedChatItem.size(), sizeof(ChatRoomItem*), ItemComareFunc, this);
	}
	ResortChatRoomList(vecSortedChatItem);

	Node* pNode = m_pChattingRoomList->FindNodeByCompareData(pChatRoomItem->nRoomID);
	if(pNode)
	{
		CString strBuf = _T("");
		CLabelUI* lastmsg = static_cast<CLabelUI*>(m_pChattingRoomList->paint_manager_.FindSubControlByName(pNode->data().list_elment_, _T("lastmsg")));
		if(lastmsg != NULL)
		{
			CString strHtml = pChatRoomItem->strLastMsg;
			strHtml.Replace(_T("\r\n"), _T(""));
			if (GET_MESSENGER_DLG()->GetMainFrame()->IsRecentChatSearchMode())
			{
				CString strSearchWord = GET_MESSENGER_DLG()->GetMainFrame()->GetSearchWord();
				CString strFullWord = pChatRoomItem->strLastMsg;
				strFullWord.Replace(_T("\r\n"), _T(""));
				int nFindIndex = strFullWord.Find(strSearchWord);
				if (nFindIndex >= 0)
				{

					CString strBuf = _T("");
					CString strLeft = strFullWord.Left(nFindIndex);
					CString strRight = strFullWord.Right(strFullWord.GetLength() - nFindIndex - strSearchWord.GetLength());
					strBuf.Format(_T("<c #919EAF>%s</c><c #187AFF>%s</c><c #919EAF>%s</c>"), strLeft, strSearchWord, strRight);
					strHtml = strBuf;
					lastmsg->SetShowHtml(true);
				}
			}
			
			lastmsg->SetText(strHtml);
			lastmsg->SetToolTip(pChatRoomItem->strLastMsg);
		}

		CButtonUI* log_button = static_cast<CButtonUI*>(m_pChattingRoomList->paint_manager_.FindSubControlByName(pNode->data().list_elment_, kLogoButtonControlName));
		if (log_button != NULL )
		{
			if (!pChatRoomItem->bGroupChat)
			{
				strBuf.Format(LoadStringFromResource(IDS_STRING_RECENT_CHAT_TOOLTIP_3), pChatRoomItem->strName.GetData(), 
					GetPresenceTip(pChatRoomItem->eStatusType), pChatRoomItem->strEMail.GetData());
				log_button->SetToolTip(strBuf);
			} 
		}

		CLabelUI* date = static_cast<CLabelUI*>(m_pChattingRoomList->paint_manager_.FindSubControlByName(pNode->data().list_elment_, _T("date")));
		if(date != NULL)
		{
			date->SetText(ParseChatTime(pChatRoomItem->tmLastTime));
		}
	}
}

void CRecentChatListManager::UpdateChatRoomList(CChatDlg* pChatDlg, BOOL bSort)
{
	//NTALK_LOG->Debug(L"void CRecentChatListManager::UpdateChatRoomList(CChatDlg* pChatDlg, BOOL bSort)");
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);

	ChatRoomItem *pChatRoomItem = NULL;
	vector<ChatRoomItem*> vecSortedChatItem;
	list<ChatRoomItem*>::iterator it;
	for (it=m_listChatItem.begin(); it!=m_listChatItem.end(); ++it)
	{
		if ((*it)->nRoomID == pChatDlg->GetRoomID())
		{
			(*it)->pChatDlg = pChatDlg;
			(*it)->strName = pChatDlg->GetAllUserNameFromList(); 
			(*it)->strUserID = pChatDlg->GetAllUserIDFromList();
			/*
				1.б¼
				2.ʱȴchatdlgȻһһӳԱ
				3.ӵһԱpChatDlg->IsGroupRoomFALSEӵڶTRUE¼Ϊ׼(*it)->bGroupChat
				3.ʱһʱ(*it)->bGroupChatΪFALSEpChatDlg->IsGroupRoomΪTRUE
				4.(*it)->bGroupChatpChatDlg->IsGroupRoom()һΪTRUE(*it)->bGroupChat = TRUE
			*/
			if ((*it)->bGroupChat || pChatDlg->IsGroupRoom())
			{
				(*it)->bGroupChat = TRUE;
			}
			else
			{
				(*it)->bGroupChat = FALSE;
			}
			(*it)->tmLastTime = pChatDlg->GetCreationTime();
			pChatRoomItem = (*it);
		}
		vecSortedChatItem.push_back(*it);
	}
	if (pChatDlg == NULL || pChatRoomItem == NULL)
	{
		return;
	}
	if (vecSortedChatItem.size() > 1)
	{
		qsort_s(&(vecSortedChatItem.at(0)), vecSortedChatItem.size(), sizeof(ChatRoomItem*), ItemComareFunc, this);
	}

	ResortChatRoomList(vecSortedChatItem);

	//updatenodeģʾݺ˳ͬʱnodeʾҲҪ¡ͻ˳ȷnodelast timeԵ
    Node* pNode = m_pChattingRoomList->FindNodeByCompareData(pChatDlg->GetRoomID());
	if(pNode)
	{
		CString strBuf = _T("");
		pChatRoomItem->strImage = GetAppDUIImgDir() + _T("\\main_profile_blank.png");
		pChatRoomItem->strLastMsg = pChatDlg->GetLastMsg();

		CButtonUI* log_button = static_cast<CButtonUI*>(m_pChattingRoomList->paint_manager_.FindSubControlByName(pNode->data().list_elment_, kLogoButtonControlName));
		CLabelUI* pLabProfile = static_cast<CLabelUI*>(m_pChattingRoomList->paint_manager_.FindSubControlByName(pNode->data().list_elment_, _T("profile")));
		if (log_button != NULL && pLabProfile != NULL)
		{
			CDuiString strImage = _T("");
			if (pChatRoomItem->bGroupChat)
			{
				strImage = GetAppDUIImgDir() + _T("\\img_main_chat_list_profile_nopic.png");
				log_button->SetToolTip(LoadStringFromResource(IDS_STRING_RECENT_CHAT_TOOLTIP_1));
			} 
			else 
			{
				strImage = GET_APP()->GetProfileImagePath(pChatRoomItem->strUserID);
				CChatUser* pChatUser = pChatDlg->GetChatModel()->GetChatUsrPtr(pChatRoomItem->strUserID);
				if (pChatUser != NULL)
				{
					strBuf.Format(LoadStringFromResource(IDS_STRING_RECENT_CHAT_TOOLTIP_3), pChatRoomItem->strName.GetData(), GetPresenceTip(pChatUser->GetStatus()), pChatUser->GetEmail());
					log_button->SetToolTip(strBuf);
				}
			}	
			log_button->SetBkImage(strImage);
			pLabProfile->SetTag((UINT_PTR)pChatRoomItem->bGroupChat);
			pChatRoomItem->strImage = strImage;
		}
		pNode->data().value = pChatRoomItem->strUserID;
		pNode->data().text_ = pChatRoomItem->strName;
		pNode->data().folder_ = pChatRoomItem->bGroupChat;

		CLabelUI* pUnreadMsg = static_cast<CLabelUI*>(m_pChattingRoomList->paint_manager_.FindSubControlByName(pNode->data().list_elment_, _T("unreadMsg")));
		if (pUnreadMsg != NULL)
		{
			LONG nUnreadMsgCount = CUnReadChatMessage::Instance()->GetCount(pChatDlg->GetRoomID());
			if (nUnreadMsgCount > 0)
			{
				pUnreadMsg->SetBkImage(_T("main_menu_badge_icon.png"));
				pUnreadMsg->SetVisible(true);
				pUnreadMsg->Invalidate();
			}
			else
			{
				pUnreadMsg->SetVisible(false);
			}
		}

		CLabelUI* nickname = static_cast<CLabelUI*>(m_pChattingRoomList->paint_manager_.FindSubControlByName(pNode->data().list_elment_, _T("nickname")));
		if(nickname != NULL)
		{
			nickname->SetText(pChatRoomItem->strName);
			if (pChatRoomItem->bGroupChat)
			{
				nickname->SetToolTip(pChatRoomItem->strName);
			}
			else
			{
				CString strUserID = pNode->data().value;
				CChatUser* pChatUser = pChatDlg->GetChatModel()->GetChatUsrPtr(strUserID);
				if (pChatUser != NULL)
				{
					strBuf.Format(LoadStringFromResource(IDS_STRING_RECENT_CHAT_TOOLTIP_3), pChatRoomItem->strName.GetData(), GetPresenceTip(pChatUser->GetStatus()), pChatUser->GetEmail());
					nickname->SetToolTip(strBuf);
				}
			}
		}
		
		CLabelUI* lastmsg = static_cast<CLabelUI*>(m_pChattingRoomList->paint_manager_.FindSubControlByName(pNode->data().list_elment_, _T("lastmsg")));
		if(lastmsg != NULL)
		{
			CString strHtml = pChatRoomItem->strLastMsg;
			strHtml.Replace(_T("\r\n"), _T(""));
			if (GET_MESSENGER_DLG()->GetMainFrame()->IsRecentChatSearchMode())
			{
				CString strSearchWord = GET_MESSENGER_DLG()->GetMainFrame()->GetSearchWord();
				CString strFullWord = pChatRoomItem->strLastMsg;
				strFullWord.Replace(_T("\r\n"), _T(""));
				int nFindIndex = strFullWord.Find(strSearchWord);
				if (nFindIndex >= 0)
				{
					CString strLocalBuf = _T("");
					CString strLeft = strFullWord.Left(nFindIndex);
					CString strRight = strFullWord.Right(strFullWord.GetLength() - nFindIndex - strSearchWord.GetLength());
					strLocalBuf.Format(_T("<c #919EAF>%s</c><c #187AFF>%s</c><c #919EAF>%s</c>"), strLeft, strSearchWord, strRight);
					strHtml = strLocalBuf;
					lastmsg->SetShowHtml(true);
				}
			}
			
			lastmsg->SetText(strHtml);
			lastmsg->SetToolTip(pChatRoomItem->strLastMsg);
		}

		CLabelUI* date = static_cast<CLabelUI*>(m_pChattingRoomList->paint_manager_.FindSubControlByName(pNode->data().list_elment_, _T("date")));
		if(date != NULL)
		{
			date->SetText(ParseChatTime(pChatRoomItem->tmLastTime));
		}
	}
}

CString CRecentChatListManager::ParseChatTime(CTime tmChat)
{
	CString strChatTime = tmChat.FormatGmt(_T("%Y.%m.%d"));;
	CTime tmNow = CTime::GetCurrentTime();
	GetLocaleTime(&tmNow);
	CString strNowTime = tmNow.FormatGmt(_T("%Y.%m.%d"));
	CTimeSpan tmSpan= tmNow - tmChat;
	LONGLONG nSecond = tmSpan.GetTotalSeconds();

	if (strChatTime == strNowTime)
	{
		if (nSecond >= 0 && nSecond < 60)
		{
			nSecond = max(nSecond, 1);
			strChatTime.Format(_T("%d"), (INT)nSecond);
			strChatTime += LoadStringFromResource(IDS_STRING_SECOND_AGO);
		}
		else if (nSecond >= 60 && nSecond < 3600)
		{
			strChatTime.Format(_T("%d"), (INT)(nSecond/60));
			strChatTime += LoadStringFromResource(IDS_STRING_MINUTE_AGO); 
		}
		else if (nSecond >= 3600 && nSecond < 3600*24)
		{
			strChatTime = tmChat.FormatGmt(_T("%H:%M"));
		}
	}
	else
	{
		strChatTime = tmChat.FormatGmt(_T("%Y.%m.%d"));
	}
	
	return strChatTime;
}

void CRecentChatListManager::ResortChatRoomList(vector <ChatRoomItem*> &vecChatItem)
{
	//ASSERT(vecChatItem.size() == m_pChattingRoomList->GetRoot()->num_children());
	CSingleLock dialogLock(&m_csChatDialogList, TRUE);
	int ChildCount = m_pChattingRoomList->GetRoot()->num_children();

	vector<ChatRoomItem*>::const_iterator itr = vecChatItem.begin();
	int roomID ;
	int index = 0;
	for(; itr != vecChatItem.end(); ++itr)
	{
		CListContainerElementUI* pListElement = NULL;
		ChatRoomItem* pr = *itr;
		roomID =pr->nRoomID;
		
		Node* foundNode = m_pChattingRoomList->FindNodeByCompareData(roomID);
		if (foundNode != NULL)
		{
            m_pChattingRoomList->GetRoot()->reinsert_child(foundNode,index); //reinsert_child()뿴cppļĺ˵
			pListElement =foundNode->data().list_elment_;
			m_pChattingRoomList->SetItemIndex(pListElement,index);
			index ++;
		}

	}
}

void CRecentChatListManager::AddChatRoomList(CChatDlg* pChatDlg)
{
	AddChatRoomList(new ChatRoomItem(pChatDlg));
	UpdateChatRoomList(pChatDlg);
}

void CRecentChatListManager::AddChatRoomList(const NE_MYROOM_DATA& myRoomData)
{
	AddChatRoomList(new ChatRoomItem(myRoomData));
}

void CRecentChatListManager::AddChatRoomList(ChatRoomItem* pItem)
{
	ASSERT((m_pChattingRoomList != NULL) && (pItem != NULL));
	if (m_pChattingRoomList != NULL)
	{
		CSingleLock dialogLock(&m_csChatDialogList, TRUE);
		pItem->UpdataChatItem();
		m_pChattingRoomList->AddChattingRoomNode(*pItem, NULL);
		m_listChatItem.push_back(pItem);
	}
	
}